<?php

/** 
 * Joomla.Plugin - System - NetzSinn Custom Fields Radius Search
 * ------------------------------------------------------------------------
 * @package     System - NetzSinn Custom Fields Radius Search
 * @author      Netzsinn.de
 * @copyright   2023 netzsinn.de
 * @license     GNU/GPLv3 <http://www.gnu.org/licenses/gpl-3.0.de.html>
 * @link        https://netzsinn.de
 * ------------------------------------------------------------------------
 */

// no direct access 
defined('_JEXEC') or die;

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

require_once JPATH_PLUGINS . '/system/netzsinn_customfieldsradiussearch/customfieldsradiussearch.helper.php';


/**
 * plg_sys_bw_customfieldsradiussearch
 */
class PlgSystemNetzsinn_customfieldsradiussearch extends CMSPlugin
{

	/**
	 * Application object
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 * @since  4.0.0
	 */
	protected $app;

	protected $activate;

	protected $path;

	protected $pathname;

	protected $max_setups = 3;

	protected $settings;

	protected $total;

	/**
	 * Load the language file on instantiation
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	/** 
	 * This is called before the Model is called. Here we change the input parameters.
	 * 
	 * @see https://docs.joomla.org/Plugin/Events/System#onAfterRoute
	 * @throws Exception
	 */
	public function onAfterRoute()
	{
		if (Factory::getApplication()->isClient('administrator')) {
			return;
		}

		// get path and query params
		$uri = Uri::getInstance();
		$this->path = ltrim($uri->getPath(), '/');


		$input = $this->app->input;
		$option = $input->getString('option', '');

		// Get the settings valid for the given URL ... or leave the Plugin
		$settings = $this->params->get('settings');

		#print_r($settings);
		$this->settings = null;
		foreach ($settings as $setting) {
			$paths = preg_split("/\r\n|\n|\r/", $setting->paths);
			#print_r($paths);
			foreach ($paths as $path) {
				// echo 'current path: ' . $this->path . '<br/>';
				// echo 'path: ' . $path . '<br/>';
				if ($this->path == ltrim($path, '/')) {
					$this->settings = new stdClass();

					$this->settings->radius = $input->getString($setting->input_param_radius);
					$this->settings->address = $input->getString($setting->input_param_address);

					$this->settings->google_api_key = $this->params->get('google_api_key');

					$coordinate = NetzSinnRadiusSearchHelper::getCoordinates($this->settings->address, $this->settings->google_api_key);

					$this->settings->lat = $coordinate['lat'];
					$this->settings->lng = $coordinate['lng'];
					$this->settings->order = $setting->order_dir;
					$this->settings->field_lat = $setting->field_lat;
					$this->settings->field_lng = $setting->field_lng;
					$this->settings->field_latlng = $setting->field_latlng;
					// $this->settings->is_fields_lat_lng = $setting->is_fields_lat_lng;

					$fields = [];
					for ($i = 1; $i <= $this->max_setups; $i++) {
						if (strlen(trim($setting->{'field' . $i . '_input'}))) {
							$field = new stdClass();
							$field->id = $setting->{'field' . $i . '_id'};
							$field->paramname = $setting->{'field' . $i . '_input'};
							$field->input = $input->getString($setting->{'field' . $i . '_input'});
							$field->comparator = $setting->{'field' . $i . '_comparator'};
							$fields[$i] = $field;
						}
					}
					$this->settings->fields = $fields;

					$this->activate = true;
					break;
				}
			}
		}
		#print_r($this->settings);

		if ($this->settings == null) {
			return true;
		}



		/**
		 * Test: https://test.e-markuspaul.de/ferienbetreuung-und-elternberatung-vor-ort-2?address=frankfurt&radius=200&latlng=50.1109221,8.6821267
		 */
		if ($this->params->get('is_debug') == true && !Factory::getUser()->guest) {
			echo '<pre>';
			echo 'radius: ' . $this->settings->radius . '<br/>';
			echo 'address: ' . $this->settings->address . '<br/>';
			echo 'lat: ' . $this->settings->lat . '<br/>';
			echo 'lng: ' . $this->settings->lng . '<br/>';
			echo 'order dir: ' . $this->settings->order . '<br/>';
			echo 'field_lat: ' . $this->settings->field_lat . '<br/>';
			echo 'field_lng: ' . $this->settings->field_lng . '<br/>';
			echo 'field_latlng: ' . $this->settings->field_latlng . '<br/>';
			// echo 'is_fields_lat_lng: ' . $this->settings->is_fields_lat_lng . '<br/>';
			$counter = 0;
			foreach ($this->settings->fields as $field) {
				$counter++;
				echo 'field ' . $counter . ' input: ' . $field->input . '<br/>';
				echo 'field ' . $counter . ' id: ' . $field->id . '<br/>';
				echo 'field ' . $counter . ' input: ' . $field->input . '<br/>';
				echo 'field ' . $counter . ' comparator: ' . $field->comparator . '<br/>';
			}
			echo '</pre>';
		}
		$counter = 0;
		foreach ($this->settings->fields as $field) {
			$counter++;

			if ($field->input == null && strlen(trim($field->input)) < 1) {
				Factory::getApplication()->enqueueMessage(Text::_('Missing additional Custom Field ' . $counter . ' input parameter: "' . $field->paramname . '"'), 'error');
				return true;
			}
		}
		if ($this->settings->radius == null || $this->settings->address == null) {
			Factory::getApplication()->enqueueMessage(Text::_('NETZSINN_CUSTOMFIELDSRADIUSSEARCH_ERROR_MISSING_INPUT'), 'error');
			return true;
		}
		$this->app->setUserState('netzsinn.customfieldsradiussearch.debug', $this->params->get('is_debug'));
		$this->app->setUserState('netzsinn.customfieldsradiussearch.maxsetups', $this->max_setups);
		$this->app->setUserState('netzsinn.customfieldsradiussearch.settings', $this->settings);

		if ($this->activate && $option == 'com_content' && !class_exists('ArticlesModel')  && !$this->app->isClient('administrator')) {
			require_once __DIR__ . '/models/com_content/ArticlesModel.php';
		}

		return;
	}

	/**
	 * onAfterRender()
	 *
	 * @return void
	 */
	public function onAfterRender()
	{
		if (Factory::getApplication()->isClient('administrator')) {
			return true;
		}

		if ($this->activate == false) {
			return true;
		}

		// Update HTML Title 
		$title = Factory::getDocument()->getTitle();
		Factory::getDocument()->setTitle($this->pathname);

		$body = $this->app->getBody();

		// Update Total Marker
		if ($this->params->get('items_total') != null &&  strcmp($this->params->get('items_total'), '') != 0) {
			$body = str_replace($this->params->get('items_total'), $this->app->getUserState('netzsinn.customfieldsfilter.items.total'), $body);
		}
		// Update Results per page Marker
		if ($this->params->get('items_perpage') != null &&  strcmp($this->params->get('items_perpage'), '') != 0) {
			$body = str_replace($this->params->get('items_perpage'), $this->app->getUserState('netzsinn.customfieldsfilter.items.perpage'), $body);
		}
		// Update Title marker 
		if ($this->params->get('path_name') != null &&  strcmp($this->params->get('path_name'), '') != 0) {
			$body = str_replace($this->params->get('path_name'), $this->pathname, $body);
		}

		// Update HTML Title 
		if ($this->params->get('items_perpage') == true) {
			#$title = Factory::getDocument()->getTitle();
			Factory::getDocument()->setTitle($this->pathname);
		}

		$this->app->setBody($body);
	}
}
